# Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

# pylint: disable=C0116, R0903, W0406
from enum import Enum


class HtcEventCodes(Enum):
    """
    Enum representing the HTC definitions of their status codes
    Workspaces doesn't need most of this detail currently, so provide some
    boolean conglomeration utilities for messaging purposes.
    """

    JobSubmitted = 0
    JobExecuting = 1
    ExecutableError = 2
    JobCheckpoint = 3
    JobEvicted = 4
    JobTerminated = 5
    JobImageUpdate = 6
    ShadowException = 7
    GenericUnused = 8
    JobAborted = 9
    JobSuspended = 10
    JobUnsuspended = 11
    JobHeld = 12
    JobReleased = 13
    ParallelNodeExecuted = 14
    ParallelNodeTerminated = 15
    PostScriptTerminated = 16
    # 17-20 Unused
    RemoteError = 21
    RemoteSocketLost = 22
    RemoteSocketRegained = 23
    RemoteReconnectFailure = 24
    GridResourceReturned = 25
    GridResourceDown = 26
    GridResourceSubmission = 27
    JobAdInformationEvent = 28
    JobRemoteStatusUnknown = 29
    JobRemoteStatusReacquired = 30
    JobStageIn = 31
    JobStageOut = 32
    JobAdAttributeUpdate = 33
    PreSkipEvent = 34
    ClusterSubmit = 35
    ClusterRemove = 36
    FactoryPaused = 37
    FactoryResumed = 38
    GenericTimeout = 39
    FileTransfer = 40

    def __init__(self, htc_value: int):
        self.htc_value = htc_value

    @property
    def is_initialization_message(self) -> bool:
        """
        Submission messages
        """
        return self.htc_value in [0, 35]

    @property
    def is_completion_message(self) -> bool:
        """
        Finished (success is another matter)
        """
        return self.htc_value in [5, 36]

    @property
    def is_informational_message(self) -> bool:
        """
        This is a bit of a catch-all, there are many subdivisions possible in the future.
        For simplicity, we lump these together to start
        """
        return self.htc_value in [1, 3, 6, 8, 14, 15, 16, 22, 23, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 37, 38, 40]

    @property
    def is_error_message(self) -> bool:
        """
        Error states (including Held)
        """
        return self.htc_value in [2, 12, 21]

    @property
    def is_resubmitting_message(self) -> bool:
        """
        This failure mode is of particular interest, since HTC will attempt to restart the job
        (potentially leading to infinite loops)
        """
        return self.htc_value == 24

    @property
    def is_pause_message(self) -> bool:
        """
        Interruptions to execution, expected that this will be followed by a continuation
        """
        return self.htc_value in [4, 7, 10]

    @property
    def is_unpause_message(self) -> bool:
        """
        Explicit restart messages, others are more indirect (JobExecuting, for instance)
        """
        return self.htc_value in [11, 13]

    @property
    def is_aborted_message(self) -> bool:
        """
        You asked, condor confirms doing the abort
        """
        return self.htc_value == 9


class WorkflowStatusMessages(Enum):
    """
    Listing of messages that the wf_monitor sends to the
    WS services for interpretation.
    """

    EXECUTING = "workflow-executing"
    CONTINUE = "workflow-continuing"
    FAILED = "workflow-failed"
    TERMINATED = "workflow-terminated"
    ABORTED = "workflow-aborted"
    PAUSED = "workflow-paused"
    RESTARTED = "workflow-restarted"
    UNEXPECTED = "unexpected-message"
